from collections.abc import Callable, Sequence
import types
from typing import overload

from . import GRB as GRB
import pyoptinterface._src.core_ext


class RawEnv:
    def __init__(self, empty: bool = False) -> None: ...

    def start(self) -> None: ...

    def raw_parameter_type(self, arg: str, /) -> int: ...

    def set_raw_parameter_int(self, arg0: str, arg1: int, /) -> None: ...

    def set_raw_parameter_double(self, arg0: str, arg1: float, /) -> None: ...

    def set_raw_parameter_string(self, arg0: str, arg1: str, /) -> None: ...

class RawModel(_RawModelBase):
    @overload
    def __init__(self) -> None: ...

    @overload
    def __init__(self, arg: RawEnv, /) -> None: ...

    def init(self, arg: RawEnv, /) -> None: ...

    def write(self, arg: str, /) -> None: ...

    def add_variable(self, domain: pyoptinterface._src.core_ext.VariableDomain = VariableDomain.Continuous, lb: float = -1e+100, ub: float = 1e+100, name: str = '') -> pyoptinterface._src.core_ext.VariableIndex: ...

    def delete_variable(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> None: ...

    def delete_variables(self, arg: Sequence[pyoptinterface._src.core_ext.VariableIndex], /) -> None: ...

    def is_variable_active(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> bool: ...

    def set_variable_bounds(self, variable: pyoptinterface._src.core_ext.VariableIndex, lb: float, ub: float) -> None: ...

    @overload
    def get_value(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> float: ...

    @overload
    def get_value(self, arg: pyoptinterface._src.core_ext.ScalarAffineFunction, /) -> float: ...

    @overload
    def get_value(self, arg: pyoptinterface._src.core_ext.ScalarQuadraticFunction, /) -> float: ...

    @overload
    def get_value(self, arg: pyoptinterface._src.core_ext.ExprBuilder, /) -> float: ...

    @overload
    def pprint(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> str: ...

    @overload
    def pprint(self, expr: pyoptinterface._src.core_ext.ScalarAffineFunction, precision: int = 4) -> str: ...

    @overload
    def pprint(self, expr: pyoptinterface._src.core_ext.ScalarQuadraticFunction, precision: int = 4) -> str: ...

    @overload
    def pprint(self, expr: pyoptinterface._src.core_ext.ExprBuilder, precision: int = 4) -> str: ...

    def set_variable_name(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, /) -> None: ...

    def set_constraint_name(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, /) -> None: ...

    @overload
    def add_linear_constraint(self, expr: pyoptinterface._src.core_ext.ScalarAffineFunction, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_linear_constraint(self, expr: pyoptinterface._src.core_ext.VariableIndex, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_linear_constraint(self, expr: pyoptinterface._src.core_ext.ExprBuilder, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_linear_constraint(self, con: pyoptinterface._src.core_ext.ComparisonConstraint, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_quadratic_constraint(self, expr: pyoptinterface._src.core_ext.ScalarQuadraticFunction, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_quadratic_constraint(self, expr: pyoptinterface._src.core_ext.ExprBuilder, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_quadratic_constraint(self, con: pyoptinterface._src.core_ext.ComparisonConstraint, name: str = '') -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_sos_constraint(self, arg0: Sequence[pyoptinterface._src.core_ext.VariableIndex], arg1: pyoptinterface._src.core_ext.SOSType, /) -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    @overload
    def add_sos_constraint(self, arg0: Sequence[pyoptinterface._src.core_ext.VariableIndex], arg1: pyoptinterface._src.core_ext.SOSType, arg2: Sequence[float], /) -> pyoptinterface._src.core_ext.ConstraintIndex: ...

    def delete_constraint(self, arg: pyoptinterface._src.core_ext.ConstraintIndex, /) -> None: ...

    def is_constraint_active(self, arg: pyoptinterface._src.core_ext.ConstraintIndex, /) -> bool: ...

    @overload
    def set_objective(self, expr: pyoptinterface._src.core_ext.ScalarQuadraticFunction, sense: pyoptinterface._src.core_ext.ObjectiveSense = pyoptinterface._src.core_ext.ObjectiveSense.Minimize) -> None: ...

    @overload
    def set_objective(self, expr: pyoptinterface._src.core_ext.ScalarAffineFunction, sense: pyoptinterface._src.core_ext.ObjectiveSense = pyoptinterface._src.core_ext.ObjectiveSense.Minimize) -> None: ...

    @overload
    def set_objective(self, expr: pyoptinterface._src.core_ext.ExprBuilder, sense: pyoptinterface._src.core_ext.ObjectiveSense = pyoptinterface._src.core_ext.ObjectiveSense.Minimize) -> None: ...

    @overload
    def set_objective(self, expr: pyoptinterface._src.core_ext.VariableIndex, sense: pyoptinterface._src.core_ext.ObjectiveSense = pyoptinterface._src.core_ext.ObjectiveSense.Minimize) -> None: ...

    @overload
    def set_objective(self, expr: float, sense: pyoptinterface._src.core_ext.ObjectiveSense = pyoptinterface._src.core_ext.ObjectiveSense.Minimize) -> None: ...

    @overload
    def cb_add_lazy_constraint(self, expr: pyoptinterface._src.core_ext.ScalarAffineFunction, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float) -> None: ...

    @overload
    def cb_add_lazy_constraint(self, expr: pyoptinterface._src.core_ext.ExprBuilder, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float) -> None: ...

    @overload
    def cb_add_user_cut(self, expr: pyoptinterface._src.core_ext.ScalarAffineFunction, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float) -> None: ...

    @overload
    def cb_add_user_cut(self, expr: pyoptinterface._src.core_ext.ExprBuilder, sense: pyoptinterface._src.core_ext.ConstraintSense, rhs: float) -> None: ...

    def optimize(self) -> None: ...

    def update(self) -> None: ...

    def version_string(self) -> str: ...

    def get_raw_model(self) -> types.CapsuleType: ...

    def set_callback(self, arg: Callable[[_RawModelBase, int], None], /) -> None: ...

    def cb_get_info_int(self, arg: int, /) -> int: ...

    def cb_get_info_double(self, arg: int, /) -> float: ...

    def cb_get_solution(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> float: ...

    def cb_get_relaxation(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> float: ...

    def cb_set_solution(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: float, /) -> None: ...

    def cb_submit_solution(self) -> float: ...

    def cb_exit(self) -> None: ...

    def raw_parameter_type(self, arg: str, /) -> int: ...

    def set_raw_parameter_int(self, arg0: str, arg1: int, /) -> None: ...

    def set_raw_parameter_double(self, arg0: str, arg1: float, /) -> None: ...

    def set_raw_parameter_string(self, arg0: str, arg1: str, /) -> None: ...

    def get_raw_parameter_int(self, arg: str, /) -> int: ...

    def get_raw_parameter_double(self, arg: str, /) -> float: ...

    def get_raw_parameter_string(self, arg: str, /) -> str: ...

    def raw_attribute_type(self, arg: str, /) -> int: ...

    def set_model_raw_attribute_int(self, arg0: str, arg1: int, /) -> None: ...

    def set_model_raw_attribute_double(self, arg0: str, arg1: float, /) -> None: ...

    def set_model_raw_attribute_string(self, arg0: str, arg1: str, /) -> None: ...

    def get_model_raw_attribute_int(self, arg: str, /) -> int: ...

    def get_model_raw_attribute_double(self, arg: str, /) -> float: ...

    def get_model_raw_attribute_string(self, arg: str, /) -> str: ...

    def get_model_raw_attribute_vector_double(self, arg0: str, arg1: int, arg2: int, /) -> list[float]: ...

    def get_model_raw_attribute_list_double(self, arg0: str, arg1: Sequence[int], /) -> list[float]: ...

    def set_variable_raw_attribute_int(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, arg2: int, /) -> None: ...

    def set_variable_raw_attribute_char(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, arg2: str, /) -> None: ...

    def set_variable_raw_attribute_double(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, arg2: float, /) -> None: ...

    def set_variable_raw_attribute_string(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, arg2: str, /) -> None: ...

    def get_variable_raw_attribute_int(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, /) -> int: ...

    def get_variable_raw_attribute_char(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, /) -> str: ...

    def get_variable_raw_attribute_double(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, /) -> float: ...

    def get_variable_raw_attribute_string(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: str, /) -> str: ...

    def set_constraint_raw_attribute_int(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, arg2: int, /) -> None: ...

    def set_constraint_raw_attribute_char(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, arg2: str, /) -> None: ...

    def set_constraint_raw_attribute_double(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, arg2: float, /) -> None: ...

    def set_constraint_raw_attribute_string(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, arg2: str, /) -> None: ...

    def get_constraint_raw_attribute_int(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, /) -> int: ...

    def get_constraint_raw_attribute_char(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, /) -> str: ...

    def get_constraint_raw_attribute_double(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, /) -> float: ...

    def get_constraint_raw_attribute_string(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: str, /) -> str: ...

    def get_normalized_rhs(self, arg: pyoptinterface._src.core_ext.ConstraintIndex, /) -> float: ...

    def set_normalized_rhs(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: float, /) -> None: ...

    def get_normalized_coefficient(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: pyoptinterface._src.core_ext.VariableIndex, /) -> float: ...

    def set_normalized_coefficient(self, arg0: pyoptinterface._src.core_ext.ConstraintIndex, arg1: pyoptinterface._src.core_ext.VariableIndex, arg2: float, /) -> None: ...

    def get_objective_coefficient(self, arg: pyoptinterface._src.core_ext.VariableIndex, /) -> float: ...

    def set_objective_coefficient(self, arg0: pyoptinterface._src.core_ext.VariableIndex, arg1: float, /) -> None: ...

    def computeIIS(self) -> None: ...

def is_library_loaded() -> bool: ...

def load_library(arg: str, /) -> bool: ...
